@@ -1,11 +1,6 @@
+from pycid.core.causal_bayesian_network import CausalBayesianNetwork  # noqa
 from pycid.core.cid import CID  # noqa
-from pycid.core.cpd import (  # noqa
-    DecisionDomain,
-    FunctionCPD,
-    RandomlySampledFunctionCPD,
-    StochasticFunctionCPD,
-    UniformRandomCPD,
-)
+from pycid.core.cpd import DecisionDomain, FunctionCPD, StochasticFunctionCPD, UniformRandomCPD  # noqa
 from pycid.core.macid import MACID  # noqa
 from pycid.core.macid_base import MACIDBase, MechanismGraph  # noqa
 from pycid.core.relevance_graph import CondensedRelevanceGraph, RelevanceGraph  # noqa
