@@ -1,17 +1,13 @@
 from __future__ import annotations
 
 import itertools
-import logging
-import random
 from functools import lru_cache
 from typing import (
-    Any,
     Callable,
     Collection,
     Dict,
     Hashable,
     Iterable,
-    Iterator,
     KeysView,
     List,
     Mapping,
@@ -19,23 +15,23 @@ from typing import (
     Set,
     Tuple,
     Union,
+    Iterator,
 )
 
 import matplotlib.cm as cm
-import matplotlib.pyplot as plt
 import networkx as nx
 import numpy as np
 from pgmpy.factors.discrete import TabularCPD
 from pgmpy.inference.ExactInference import BeliefPropagation
-from pgmpy.models import BayesianModel
 
-from pycid.core.cpd import DecisionDomain, FunctionCPD, ParentsNotReadyException, UniformRandomCPD
+from pycid.core.causal_bayesian_network import CausalBayesianNetwork
+from pycid.core.cpd import DecisionDomain, FunctionCPD, Outcome, UniformRandomCPD
 from pycid.core.relevance_graph import RelevanceGraph
 
 AgentLabel = Hashable  # Could be a TypeVar instead but that might be overkill
 
 
-class MACIDBase(BayesianModel):
+class MACIDBase(CausalBayesianNetwork):
     """Base structure of a Multi-Agent Causal Influence Diagram.
 
     Attributes
@@ -71,7 +67,7 @@ class MACIDBase(BayesianModel):
         agent_utilities: The utility nodes of each agent.
             A mapping of agent label => node labels.
         """
-        super().__init__(ebunch=edges)
+        super().__init__(edges=edges)
 
         self.agent_decisions = {agent: list(nodes) for agent, nodes in agent_decisions.items()}
         self.agent_utilities = {agent: list(nodes) for agent, nodes in agent_utilities.items()}
@@ -79,8 +75,6 @@ class MACIDBase(BayesianModel):
         self.decision_agent = {node: agent for agent, nodes in self.agent_decisions.items() for node in nodes}
         self.utility_agent = {node: agent for agent, nodes in self.agent_utilities.items() for node in nodes}
 
-        self._cpds_to_add: Dict[str, TabularCPD] = {}
-
     @property
     def decisions(self) -> KeysView[str]:
         """The set of all decision nodes"""
@@ -96,88 +90,47 @@ class MACIDBase(BayesianModel):
         """The set of all agents"""
         return self.agent_utilities.keys()
 
-    def remove_edge(self, u: str, v: str) -> None:
-        super().remove_edge(u, v)
-        # remove_edge can be called during __init__ when cpds is not yet defined
-        if not hasattr(self, "cpds"):
-            return
-        cpd = self.get_cpds(v)
-        if isinstance(cpd, UniformRandomCPD):
-            self.add_cpds(cpd)
-
-    def add_edge(self, u: str, v: str) -> None:
-        super().add_edge(u, v)
-        # add_edge can be called during __init__ when cpds is not yet defined
-        if not hasattr(self, "cpds"):
-            return
-        cpd = self.get_cpds(v)
-        if isinstance(cpd, UniformRandomCPD):
-            self.add_cpds(cpd)
-
     def make_decision(self, node: str, agent: AgentLabel = 0) -> None:
         """"Turn a chance or utility node into a decision node."""
+        self.make_chance(node)
+        self.agent_decisions[agent].append(node)
+        self.decision_agent[node] = agent
         cpd = self.get_cpds(node)
-        if cpd is None:
-            raise ValueError(f"node {node} has not yet been assigned a domain.")
-        elif isinstance(cpd, DecisionDomain):
-            # Already a decision
-            pass
-        else:
-            cpd_new = DecisionDomain(node, cpd.state_names[node])
-            self.agent_decisions[agent].append(node)
-            self.decision_agent[node] = agent
-            self.add_cpds(cpd_new)
+        if cpd and not isinstance(cpd, DecisionDomain):
+            self.add_cpds(DecisionDomain(node, cpd.state_names[node]))
+
+    def make_utility(self, node: str, agent: AgentLabel = 0) -> None:
+        """"Turn a chance or utility node into a decision node."""
+        self.make_chance(node)
+        self.agent_utilities[agent].append(node)
+        self.utility_agent[node] = agent
 
     def make_chance(self, node: str) -> None:
         """Turn a decision node into a chance node."""
-        agent = self.decision_agent.pop(node)
-        self.agent_decisions[agent].remove(node)
+        if node not in self.nodes():
+            raise KeyError(f"The node {node} is not in the (MA)CID")
+        elif node in set(self.decisions):
+            agent = self.decision_agent.pop(node)
+            self.agent_decisions[agent].remove(node)
+        elif node in set(self.utilities):
+            agent = self.utility_agent.pop(node)
+            self.agent_utilities[agent].remove(node)
 
     def add_cpds(self, *cpds: TabularCPD) -> None:
         """
-        Add the given CPDs and initialize FunctionCPDs, UniformRandomCPDs etc
+        Add the given CPDs and initialize them.
+
+        A CPD can be a TabularCPD, FunctionCPD, UniformRandomCPD, DecisionDomain, or a StochasticFunctionCPD.
         """
 
-        # Add each cpd to self._cpds_to_add after doing some checks
+        # For MACIDs and CIDs we need to do an extra check
         for cpd in cpds:
-            assert cpd.variable in self.nodes
-            assert isinstance(cpd, TabularCPD)
             if isinstance(cpd, DecisionDomain) and cpd.variable not in self.decisions:
                 raise ValueError(f"trying to add DecisionDomain to non-decision node {cpd.variable}")
-            if hasattr(cpd, "check_parents"):
-                cpd.check_parents(self)
-            self._cpds_to_add[cpd.variable] = cpd
-
-        # Initialize CPDs in topological order. Call super().add_cpds if initialized
-        # successfully. Otherwise leave in self._cpds_to_add.
-        for var in nx.topological_sort(self):
-            if var in self._cpds_to_add:
-                cpd_to_add = self._cpds_to_add[var]
-                if hasattr(cpd_to_add, "initialize_tabular_cpd"):
-                    try:
-                        cpd_to_add.initialize_tabular_cpd(self)
-                    except ParentsNotReadyException:
-                        pass
-                if hasattr(cpd_to_add, "values"):  # cpd_to_add has been initialized
-                    # if the state_names have changed, remember to update all descendants:
-                    previous_cpd = self.get_cpds(var)
-                    if previous_cpd and previous_cpd.state_names[var] != cpd_to_add.state_names[var]:
-                        for descendant in nx.descendants(self, var):
-                            if descendant not in self._cpds_to_add and self.get_cpds(descendant):
-                                self._cpds_to_add[descendant] = self.get_cpds(descendant)
-
-                    # add cpd to BayesianModel, and remove it from _cpds_to_add
-                    #
-                    # pgmpy produces warnings when overwriting an existing CPD
-                    # It writes warnings directly to the 'root' context so
-                    # to suppress those we disable warnings for all loggers
-                    logging.disable(logging.WARN)
-                    super().add_cpds(cpd_to_add)
-                    logging.disable(logging.NOTSET)  # Unset
-                    del self._cpds_to_add[var]
+        super().add_cpds(*cpds)
 
     def query(
-        self, query: Iterable[str], context: Dict[str, Any], intervention: Dict[str, Any] = None
+        self, query: Iterable[str], context: Dict[str, Outcome], intervention: Dict[str, Outcome] = None
     ) -> BeliefPropagation:
         """Return P(query|context, do(intervention))*P(context | do(intervention)).
 
@@ -188,33 +141,29 @@ class MACIDBase(BayesianModel):
         ----------
         query: A set of nodes to query.
 
-        context: Node values to condition upon. A dictionary mapping of node => value.
+        context: Node values to condition upon. A dictionary mapping of node => outcome.
 
-        intervention: Interventions to apply. A dictionary mapping node => value.
+        intervention: Interventions to apply. A dictionary mapping node => outcome.
         """
 
-        for variable, value in context.items():
-            if value not in self.get_cpds(variable).state_names[variable]:
-                raise ValueError(f"The value {value} is not in the state_names of {variable}")
+        for variable, outcome in context.items():
+            if outcome not in self.get_cpds(variable).state_names[variable]:
+                raise ValueError(f"The outcome {outcome} is not in the domain of {variable}")
 
         if intervention is None:
             intervention = {}
 
-        # First apply the intervention, if any
-        if intervention:
-            cid = self.copy()
-            cid.intervene(intervention)
-        else:
-            cid = self
-
         # Check that strategically relevant decisions have a policy specified
-        mech_graph = MechanismGraph(cid)
-        for decision in cid.decisions:
+        mech_graph = MechanismGraph(self)
+        for intervention_var in intervention:
+            for parent in self.get_parents(intervention_var):
+                mech_graph.remove_edge(parent, intervention_var)
+        for decision in self.decisions:
             for query_node in query:
                 if mech_graph.is_active_trail(
                     decision + "mec", query_node, observed=list(context.keys()) + list(intervention.keys())
                 ):
-                    cpd = cid.get_cpds(decision)
+                    cpd = self.get_cpds(decision)
                     if not cpd:
                         raise ValueError(f"no DecisionDomain specified for {decision}")
                     elif isinstance(cpd, DecisionDomain):
@@ -222,89 +171,10 @@ class MACIDBase(BayesianModel):
                             f"P({query}|{context}, do({intervention})) depends on {decision}, but no policy imputed"
                         )
 
-        # query fails if graph includes nodes not in moralized graph, so we remove them
-        # cid = self.copy()
-        # mm = MarkovModel(cid.moralize().edges())
-        # for node in self.nodes:
-        #     if node not in mm.nodes:
-        #         cid.remove_node(node)
-        # filtered_context = {k:v for k,v in context.items() if k in mm.nodes}
-
-        updated_state_names = {}
-        for v in query:
-            cpd = cid.get_cpds(v)
-            updated_state_names[v] = cpd.state_names[v]
-
-        # Make a copy of self and revise the context without state_names (to handle a pgmpy bug),
-        copy = cid.copy_without_cpds()
-        for cpd in cid.cpds:
-            evidence = cpd.variables[1:] if len(cpd.variables) > 1 else None
-            evidence_card = cpd.cardinality[1:] if len(cpd.variables) > 1 else None
-            copy.add_cpds(TabularCPD(cpd.variable, cpd.variable_card, cpd.get_values(), evidence, evidence_card))
-        revised_context = {  # state_names are switched to their state number
-            variable: self.get_cpds(variable).name_to_no[variable][value] for variable, value in context.items()
-        }
-
-        bp = BeliefPropagation(copy)
-        # TODO: check for probability 0 queries
-
-        with np.errstate(invalid="ignore"):  # Suppress numpy warnings for 0/0
-            factor = bp.query(query, revised_context, show_progress=False)
-        factor.state_names = updated_state_names  # reintroduce the state_names
-        return factor
-
-    def intervene(self, intervention: Dict[str, Any]) -> None:
-        """Given a dictionary of interventions, replace the CPDs for the relevant nodes.
-
-        Soft interventions can be achieved by using self.add_cpds() directly.
-
-        Parameters
-        ----------
-        intervention: Interventions to apply. A dictionary mapping node => value.
-        """
-        for variable in intervention:
-            for p in self.get_parents(variable):  # remove ingoing edges
-                self.remove_edge(p, variable)
-            self.add_cpds(FunctionCPD(variable, lambda: intervention[variable]))
-
-    def expected_value(
-        self,
-        variables: Iterable[str],
-        context: Dict[str, Any],
-        intervene: Dict[str, Any] = None,
-    ) -> List[float]:
-        """Compute the expected value of a real-valued variable for a given context,
-        under an optional intervention
-
-        Parameters
-        ----------
-        variables: A set of variables to evaluate.
-
-        context: Node values to condition upon. A dictionary mapping of node => value.
-
-        intervention: Interventions to apply. A dictionary mapping node => value.
-        """
-        factor = self.query(variables, context, intervention=intervene)
-        factor.normalize()  # make probs add to one
-
-        ev = np.array([0.0 for _ in factor.variables])
-        for idx, prob in np.ndenumerate(factor.values):
-            # idx contains the information about the value each variable takes
-            # we use state_names to convert index into the actual value of the variable
-            ev += prob * np.array(
-                [factor.state_names[variable][idx[var_idx]] for var_idx, variable in enumerate(factor.variables)]
-            )
-            if np.isnan(ev).any():
-                raise RuntimeError(
-                    "query {} | {} generated Nan from idx: {}, prob: {}, \
-                                consider imputing a random decision".format(
-                        variables, context, idx, prob
-                    )
-                )
-        return ev.tolist()  # type: ignore
+        return super().query(query, context, intervention)
 
     def expected_utility(
-        self, context: Dict[str, Any], intervene: Dict[str, Any] = None, agent: AgentLabel = 0
+        self, context: Dict[str, Outcome], intervention: Dict[str, Outcome] = None, agent: AgentLabel = 0
     ) -> float:
         """Compute the expected utility of an agent for a given context and optional intervention
 
@@ -320,7 +190,7 @@ class MACIDBase(BayesianModel):
 
         agent: Evaluate the utility of this agent.
         """
-        return sum(self.expected_value(self.agent_utilities[agent], context, intervene=intervene))
+        return sum(self.expected_value(self.agent_utilities[agent], context, intervention=intervention))
 
     def get_valid_order(self, nodes: Optional[Iterable[str]] = None) -> List[str]:
         """Get a topological order of the specified set of nodes (this may not be unique).
@@ -397,52 +267,44 @@ class MACIDBase(BayesianModel):
                 return False
         return True
 
-    def pure_decision_rules(self, decision: str) -> List[FunctionCPD]:
+    def pure_decision_rules(self, decision: str) -> Iterator[FunctionCPD]:
         """Return a list of the decision rules available at the given decision"""
 
-        cpd: TabularCPD = self.get_cpds(decision)
-        evidence_card = cpd.cardinality[1:]
-        parents = cpd.variables[1:]
-        state_names = cpd.state_names[decision]
+        domain = self.get_cpds(decision).state_names[decision]
+        parents = self.get_parents(decision)
+        parent_cardinalities = [self.get_cardinality(parent) for parent in parents]
 
-        parent_values = []
-        for p in parents:
-            p_cpd = self.get_cpds(p)
-            parent_values.append(p_cpd.state_names[p])
-        pv_list = list(itertools.product(*parent_values))
-        possible_arguments = [{p.lower(): pv[i] for i, p in enumerate(parents)} for pv in pv_list]
+        # We begin by representing each possible decision rule as a tuple of outcomes, with
+        # one element for each possible decision context
+        number_of_decision_contexts = int(np.product(parent_cardinalities))
+        functions_as_tuples = itertools.product(domain, repeat=number_of_decision_contexts)
 
-        # We begin by representing each possible decision as a list values, with length
-        # equal the number of decision contexts
-        functions_as_lists = list(itertools.product(state_names, repeat=np.product(len(possible_arguments))))
-
-        def arg2idx(parent_values: Dict[str, Any]) -> int:
+        def arg2idx(pv: Dict[str, Outcome]) -> int:
             """Convert a decision context into an index for the function list"""
             idx = 0
-            for i, p in enumerate(parents):
-                name_to_no: Dict[Any, int] = self.get_cpds(p).name_to_no[p]
-                idx += name_to_no[parent_values[p.lower()]] * np.product(evidence_card[:i])
-            assert 0 <= idx <= len(functions_as_lists)
+            for i, parent in enumerate(parents):
+                name_to_no: Dict[Outcome, int] = self.get_cpds(parent).name_to_no[parent]
+                idx += name_to_no[pv[parent.lower()]] * int(np.product(parent_cardinalities[:i]))
+            assert 0 <= idx <= number_of_decision_contexts
             return idx
 
-        function_cpds: List[FunctionCPD] = []
-        for func_list in functions_as_lists:
+        for func_list in functions_as_tuples:
 
             def produce_function(early_eval_func_list: tuple = func_list) -> Callable:
+                # using a default argument is a trick to get func_list to evaluate early
                 return lambda **parent_values: early_eval_func_list[arg2idx(parent_values)]
 
-            function_cpds.append(FunctionCPD(decision, produce_function(), state_names=cpd.state_names))
-        return function_cpds
+            yield FunctionCPD(decision, produce_function(), domain=domain)
 
-    def pure_strategies(self, decision_nodes: Iterable[str]) -> Iterator[Tuple[FunctionCPD, ...]]:
+    def pure_policies(self, decision_nodes: Iterable[str]) -> Iterator[Tuple[FunctionCPD, ...]]:
         """
         Iterate over all of an agent's pure policies in this subgame.
         """
         possible_dec_rules = list(map(self.pure_decision_rules, decision_nodes))
         return itertools.product(*possible_dec_rules)
 
-    def optimal_pure_strategies(self, decisions: Iterable[str], eps: float = 1e-8) -> List[Tuple[FunctionCPD, ...]]:
-        """Find all optimal strategies for a given set of decisions.
+    def optimal_pure_policies(self, decisions: Iterable[str], eps: float = 1e-8) -> List[Tuple[FunctionCPD, ...]]:
+        """Find all optimal policies for a given set of decisions.
 
         - All decisions must belong to the same agent.
         - eps is the margin of error when comparing utilities for equality.
@@ -464,23 +326,23 @@ class MACIDBase(BayesianModel):
             ):
                 macid.impute_random_decision(d)
 
-        optimal_strategies = []
+        optimal_policies = []
         max_utility = float("-inf")
-        for strategy in macid.pure_strategies(decisions):
-            macid.add_cpds(*strategy)
+        for policy in macid.pure_policies(decisions):
+            macid.add_cpds(*policy)
             expected_utility = macid.expected_utility({}, agent=agent)
             if abs(expected_utility - max_utility) < eps:
-                optimal_strategies.append(strategy)
+                optimal_policies.append(policy)
             elif expected_utility > max_utility:
-                optimal_strategies = [strategy]
+                optimal_policies = [policy]
                 max_utility = expected_utility
-        return optimal_strategies
+        return optimal_policies
 
     def optimal_pure_decision_rules(self, decision: str) -> List[FunctionCPD]:
         """
         Return a list of all optimal decision rules for a given decision
         """
-        return [strategy[0] for strategy in self.optimal_pure_strategies([decision])]
+        return [policy[0] for policy in self.optimal_pure_policies([decision])]
 
     def impute_random_decision(self, d: str) -> None:
         """Impute a random policy to the given decision node"""
@@ -496,39 +358,53 @@ class MACIDBase(BayesianModel):
         for d in self.decisions:
             self.impute_random_decision(d)
 
+    def remove_all_decision_rules(self) -> None:
+        """ Remove the decision rules from all decisions in the (MA)CID"""
+        for d in self.decisions:
+            cpd = self.get_cpds(d)
+            if cpd is None:
+                raise ValueError(f"decision {d} has not yet been assigned a domain.")
+            elif isinstance(cpd, DecisionDomain):
+                continue
+            else:
+                self.add_cpds(DecisionDomain(d, cpd.domain))
+
     def impute_optimal_decision(self, d: str) -> None:
         """Impute an optimal policy to the given decision node"""
-        self.add_cpds(random.choice(self.optimal_pure_decision_rules(d)))
-        # self.impute_random_decision(d)
-        # cpd = self.get_cpds(d)
-        # parents = cpd.variables[1:]
-        # idx2name = cpd.no_to_name[d]
-        # utility_nodes = self.agent_utilities[self.decision_agent[d]]
-        # descendant_utility_nodes = list(set(utility_nodes).intersection(nx.descendants(self, d)))
-        # new = self.copy()  # using a copy "freezes" the policy so it doesn't adapt to future interventions
-        #
-        # @lru_cache(maxsize=1000)
-        # def opt_policy(*parent_values: tuple) -> Any:
-        #     nonlocal descendant_utility_nodes
-        #     context: Dict[str, Any] = {parents[i]: parent_values[i] for i in range(len(parents))}
-        #     eu = []
-        #     for d_idx in range(new.get_cardinality(d)):
-        #         context[d] = idx2name[d_idx]
-        #         eu.append(sum(new.expected_value(descendant_utility_nodes, context)))
-        #     return idx2name[np.argmax(eu)]
-        #
-        # self.add_cpds(FunctionCPD(d, opt_policy, parents, state_names=cpd.state_names, label="opt"))
+        # self.add_cpds(random.choice(self.optimal_pure_decision_rules(d)))
+        self.impute_random_decision(d)
+        cpd = self.get_cpds(d)
+        parents = cpd.variables[1:]
+        idx2name = cpd.no_to_name[d]
+        utility_nodes = self.agent_utilities[self.decision_agent[d]]
+        descendant_utility_nodes = list(set(utility_nodes).intersection(nx.descendants(self, d)))
+        new = self.copy()  # using a copy "freezes" the policy so it doesn't adapt to future interventions
+
+        @lru_cache(maxsize=1000)
+        def opt_policy(**parent_values: Outcome) -> Outcome:
+            nonlocal descendant_utility_nodes
+            context: Dict[str, Outcome] = {p: parent_values[p.lower()] for p in parents}
+            eu = []
+            for d_idx in range(new.get_cardinality(d)):
+                context[d] = idx2name[d_idx]
+                eu.append(sum(new.expected_value(descendant_utility_nodes, context)))
+            return idx2name[np.argmax(eu)]
+
+        self.add_cpds(FunctionCPD(d, opt_policy, domain=cpd.state_names[d], label="opt"))
 
     def impute_conditional_expectation_decision(self, d: str, y: str) -> None:
         """Imputes a policy for d = the expectation of y conditioning on d's parents"""
         # TODO: Move to analyze, as this is not really a core feature?
-        parents = self.get_parents(d)
+        parents: List[str] = self.get_parents(d)
         new = self.copy()
 
         @lru_cache(maxsize=1000)
-        def cond_exp_policy(**pv: tuple) -> float:
-            context = {p: pv[p.lower()] for p in parents}
-            return new.expected_value([y], context)[0]
+        def cond_exp_policy(**pv: Outcome) -> float:
+            if y.lower() in pv:
+                return pv[y.lower()]  # type: ignore
+            else:
+                context = {p: pv[p.lower()] for p in parents}
+                return new.expected_value([y], context)[0]
 
         self.add_cpds(FunctionCPD(d, cond_exp_policy, label="cond_exp({})".format(y)))
 
@@ -548,13 +424,6 @@ class MACIDBase(BayesianModel):
             agent_utilities=self.agent_utilities,
         )
 
-    def copy(self) -> MACIDBase:
-        """copy the MACIDBase object"""
-        model_copy = self.copy_without_cpds()
-        if self.cpds:
-            model_copy.add_cpds(*[cpd.copy() for cpd in self.cpds])
-        return model_copy
-
     def _get_color(self, node: str) -> Union[np.ndarray, str]:
         """
         Assign a unique colour to each new agent's decision and utility nodes
@@ -569,7 +438,7 @@ class MACIDBase(BayesianModel):
             except KeyError:
                 agent = None
         if agent is not None:
-            color: np.ndarray = colors[agents.index(agent)]
+            color: np.ndarray = colors[[agents.index(agent)]]
             return color
         else:
             return "lightgray"  # chance node
@@ -582,59 +451,6 @@ class MACIDBase(BayesianModel):
         else:
             return "o"
 
-    def _get_label(self, node: str) -> Any:
-        cpd = self.get_cpds(node)
-        if hasattr(cpd, "label"):
-            return cpd.label
-        elif hasattr(cpd, "__name__"):
-            return cpd.__name__
-        else:
-            return ""
-
-    def draw(
-        self,
-        node_color: Callable[[str], str] = None,
-        node_shape: Callable[[str], str] = None,
-        node_label: Callable[[str], str] = None,
-    ) -> None:
-        """
-        Draw the MACID or CID.
-        """
-        color = node_color if node_color else self._get_color
-        shape = node_shape if node_shape else self._get_shape
-        label = node_label if node_label else self._get_label
-        layout = nx.kamada_kawai_layout(self)
-        label_dict = {node: label(node) for node in self.nodes}
-        pos_higher = {}
-        for k, v in layout.items():
-            if v[1] > 0:
-                pos_higher[k] = (v[0] - 0.1, v[1] - 0.2)
-            else:
-                pos_higher[k] = (v[0] - 0.1, v[1] + 0.2)
-        nx.draw_networkx(self, pos=layout, node_size=800, arrowsize=20)
-        nx.draw_networkx_labels(self, pos_higher, label_dict)
-        for node in self.nodes:
-            nx.draw_networkx(
-                self.to_directed().subgraph([node]),
-                pos=layout,
-                node_size=800,
-                arrowsize=20,
-                node_color=color(node),
-                node_shape=shape(node),
-            )
-        plt.show()
-
-    def draw_property(self, node_property: Callable[[str], bool], color: str = "red") -> None:
-        """Draw a CID with the nodes satisfying node_property highlighted"""
-
-        def node_color(node: str) -> Any:
-            if node_property(node):
-                return color
-            else:
-                return self._get_color(node)
-
-        self.draw(node_color=node_color)
-
 
 class MechanismGraph(MACIDBase):
     """A mechanism graph has an extra parent node+"mec" for each node"""
